#! /bin/bash

# mkdocs-et.sh                  CC BY-SA Edouard.Thiel@univ-amu.fr - 25/04/2024
#
# A configurable bash script to install mkdocs in a virtualenv, to generate 
# a filtered subset of markdown pages with mkdocs, and to publish with rsync.
#
# See https://pageperso.lis-lab.fr/~edouard.thiel/mkdocs-et/
#
# WARNING: THIS PROGRAM COMES WITHOUT WARRANTY, USE AT YOUR OWN RISK!

VERSION="1.5"
DEBUG=0

#------------------------ C O N F I G U R A T I O N ---------------------------

# Base configuration
SKEL_DIR="skel-mkdocs-et"
LOCAL_CONF="conf-mkdocs-et.sh"

# Directory for the virtualenv
VENV_DIR="$HOME/.local/share/venv-mkdocs-et"

# Temp dir for mkdocs; must be on same partition then .md sources because
# mkdocs changes detection needs hard links.
TMP_DIR="$HOME/.cache/tmp-mkdocs-et-$$"

# List of python packages to be installed in the virtualenv
VENV_PACKAGES=(mkdocs-material tornado==5.1.1 mkdocs-mermaid2-plugin \
               mkdocs-macros-plugin mkdocs-glightbox)

# Apache .htaccess file
HTACCESS_SRC="htaccess"
HTACCESS_DEST=".htaccess"

# Duration between two checks of config change, in seconds
MONITOR_PERIOD=10

# Comment this out to disable terminal colors
ENABLE_COLORS=1


#---------------------- T E R M I N A L   C O L O R S -------------------------

# https://en.wikipedia.org/wiki/ANSI_escape_code#3-bit_and_4-bit
COLOR_MSG="1;94"    # blue
COLOR_ERR="1;91"    # red


set_term_color() # fd color
{
    local fd=$1 color=$2
    test -t $fd -a "$ENABLE_COLORS" = "1" && echo -e -n "\e[${color}m"
}


unset_term_color() # fd
{
    local fd=$1
    test -t $fd -a "$ENABLE_COLORS" = "1" && echo -e -n "\e[m"
}


echo_ext() # fd ncolor "$@"
{
    local fd=$1 ncolor=$2
    shift 2
    {
        set_term_color $fd $ncolor
        echo "$@"
        unset_term_color $fd
    } >& $fd
}


echo_col() # "$@"
{
    echo_ext 1 "$COLOR_MSG" "$@"
}


echo_err() # "$@"
{
    echo_ext 2 "$COLOR_ERR" "$@"
}


#------------------------- B A S H   V E R S I O N ----------------------------

check_bash_version()
{
    local a=4 b=3   # bash version >= a.b

    (( BASH_VERSINFO[0] > a || \
      (BASH_VERSINFO[0] == a && BASH_VERSINFO[1] >= b) )) || {
        echo_err "Error: bash version >= $a.$b expected."
        return 1
    }
}


#-------------------------- V I R T U A L   E N V -----------------------------

enhance_material_with_nav_title()
{
    local f n e1 e2

    echo_col "Enhancing package Material with nav_title..."

    # Add the possibility of a navigation title
    # see https://github.com/squidfunk/mkdocs-material/issues/2550

    f=$(find "$VENV_DIR" -name "nav.html" 2>/dev/null | grep "material/templates/partials")
    n=$(wc -l <<< "$f")

    if test -z "$f" -o "$n" -ne 1 ; then
        echo_err "Error: could not find nav.html in material"
        return
    fi
    echo "Checking \"$f\""

    e1="^[ ]*{{[ ]*config.site_name[ ]*}}"
    e2="    {# In replacement of {{ config.site_name }} by Mkdocs-et #}\n\
    {% if nav_title|trim %}{{ nav_title }}{% else %}{{ config.site_name }}{% endif %}"
    if grep -E "$e1" "$f" >/dev/null 2>&1; then
        echo "Save $f -> .back"
        cp "$f" "$f.back"
        echo "Modify config.site_name and allow nav_title"
        sed -i "s/$e1/$e2/" "$f"
        echo "done; diff is"
        diff "$f.back" "$f"
    else
        echo "Could not find config.site_name"
    fi
}


enhance_material_with_extracopyright()
{
    local f n e1 e2

    echo_col "Enhancing package Material with extracopyright..."

    # Add the possibility of an extracopyright
    # see https://github.com/squidfunk/mkdocs-material/issues/2508

    f=$(find "$VENV_DIR" -name "copyright.html" 2>/dev/null | 
        grep "material/templates/partials")
    n=$(wc -l <<< "$f")

    if test -z "$f" -o "$n" -ne 1 ; then
        echo_err "Error: could not find copyright.html in material"
        return
    fi

    echo "Save $f -> .back"
    cp "$f" "$f.back"
    echo "Modify $f ..."
    if sed -i '/Material for MkDocs/ {
        N
        s/<\/a>/<\/a>\n    {# Added by Mkdocs-et #}\n    {{ extracopyright }}/
      }' "$f"
    then
        echo "done; diff is"
        diff "$f.back" "$f"
    else
        echo "Could not find 'Material for MkDocs\n</a>'"
    fi
}


create_venv()
{
    local a=3 b=6  # python version >= a.b

    test_venv && { 
        echo_err "Error: virtualenv already exists."
        return 1
    }

    python3 --version >/dev/null 2>&1 || { 
        echo_err "Error: python3 interpreter is not installed;"
        echo_err "please install python3 (>= $a.$b), then rerun the script."
        return 1
    }

    python3 -c "import sys; exit(1) if sys.version_info.major < $a or \
                (sys.version_info.major == $a and sys.version_info.minor < $b) \
                else exit(0)" >/dev/null 2>&1 || { 
        echo_err "Error: python3 version >= $a.$b expected."
        return 1
    }

    virtualenv --version >/dev/null 2>&1 || { 
        echo_err "Error: virtualenv is not installed;"
        echo_err "please install virtualenv, then rerun the script."
        return 1
    }

    echo_col "Creating virtualenv $VENV_DIR ..."
    virtualenv -p python3 "$VENV_DIR" || exit 1

    echo_col "Activating virtualenv..."
    source "$VENV_DIR/bin/activate"

    echo_col "Installing packages..."
    pip3 install --upgrade pip
    pip3 install "${VENV_PACKAGES[@]}"

    # Fragile: might evolve with new Material versions
    enhance_material_with_nav_title
    enhance_material_with_extracopyright

    list_venv

    set_term_color 1 "$COLOR_MSG"
    cat <<- EOT
	virtualenv created in "$VENV_DIR".
	Warning: do not rename or move this directory, because paths are hard-coded
	inside the virtualenv; use arguments remove-venv/create-venv instead.
	EOT
    unset_term_color 1
}


run_venv()
{
    test_venv || { 
        echo_err "Error: virtualenv does not exists."
        echo_err "First create a virtualenv with: $0 create-venv"
        return 1
    }
    echo "source \"$VENV_DIR/bin/activate\""
    source "$VENV_DIR/bin/activate"
}


remove_venv()
{
    if ! test_venv ; then
        if test -d "$VENV_DIR" ; then
            echo_err "Error: the virtualenv directory exists but is not functionnal;"
            echo_err "to remove, type: rm -rf \"$VENV_DIR\""
        else
            echo_err "Error: the virtualenv does not exist."
        fi
        return 1
    fi

    echo_col "Removing virtualenv $VENV_DIR ..."
    rm -rf "$VENV_DIR"
    echo_col "done."
}


list_venv()
{
    echo
    echo_col "Installed packages:"
    pip3 list
}


test_venv()
{
    test -f "$VENV_DIR/bin/activate"
}


#---------------------------- F I L T E R   N A V -----------------------------

is_integer() # s
{
    local s=$1
    [[ -n "$s" && "$s" != *[^0-9]* ]]
}


get_number_from_fname() # s
{
    local s=$1 n
    s=${s%%.*}
    s=${s##*/}
    n=$(echo "$s" | tr -dc 0-9)
    expr $n + 0       # remove 0 prefixes to avoid them seen as octal
}


get_maxn_for_opt() # opt
{
    local opt=$1 maxn

    $FLAG_ALL && { echo "-1" ; return ;}

    maxn=${ARG_FILTERS[$opt]}
    is_integer "$maxn" || { echo "0" ; return ;}
    echo $maxn
}


make_nav()
{
    local item nt fname short_title nav_filter opt filter maxn n c

    # insert NAV_BEGIN files.
    # the ":" characters are translated to html "&#58;" to make PyYAML happy
    for item in "${NAV_BEGIN[@]}" ; do
        IFS=${item:0:1} read fname short_title <<< "${item:1}"
        echo "  - ${short_title//:/&#58;}: '$fname'"
    done

    # compute dict titles from NAV_TITLES
    declare -A titles   
    for nt in "${NAV_TITLES[@]}" ; do
        IFS=${nt:0:1} read fname short_title <<< "${nt:1}"
        test -n "$fname" && titles[$fname]=$short_title
    done

    # insert filtered files using ARG_FILTERS and FLAG_ALL
    for nav_filter in "${NAV_FILTERS[@]}" ; do
        opt="--${nav_filter%:*}"
        filter=${nav_filter#*:}
        maxn=$(get_maxn_for_opt "$opt")
 
        # filter files in filter, using maxn
        for fname in $filter ; do
            # comment out the nav item when filtered
            n=$(get_number_from_fname "$fname")
            c=" " ; ((maxn >= 0 && n > maxn)) && c="#"
            short_title=${titles[$fname]}
            test -z "$short_title" && short_title=${fname%%.*}
            echo "$c - ${short_title//:/&#58;}: '$fname'"
        done
    done

    # insert NAV_END files
    for item in "${NAV_END[@]}" ; do
        IFS=${item:0:1} read fname short_title <<< "${item:1}"
        echo "  - ${short_title//:/&#58;}: '$fname'"
    done

    echo
}


create_tmp_mkdocs_yml()
{
    # concat yml files
    cp -f "$SKEL_DIR/mkdocs.yml" "$TMP_DIR/"

    echo -e "\n# Generated nav block" >> "$TMP_DIR/mkdocs.yml"

    if ! grep '^nav:' "$SKEL_DIR/mkdocs.yml" >/dev/null 2>&1 ; then
        echo "nav:" >> "$TMP_DIR/mkdocs.yml"
    fi

    echo_col "Creating filtered nav section in $TMP_DIR/mkdocs.yml..."
    make_nav $add_nav | tee -a "$TMP_DIR/mkdocs.yml"
}


#-------------------------- T M P   D I R E C T O R Y -------------------------

clean_tmp_dir()
{
    echo_col "Cleaning $TMP_DIR ..."
    test -d "$TMP_DIR" || return
    if ((DEBUG == 0)) ; then
        rm -rf "$TMP_DIR"
    else
        echo_err "DEBUG mode: directory not removed!!"
    fi
}


create_tmp_dir()
{
    local f g item fname short_title nav_filter opt filter maxn n

    test -d "$SKEL_DIR" || { 
        echo_err "Error: $SKEL_DIR/ is missing."
        return 1
    }
    echo_col "Creating $TMP_DIR ..."
    trap clean_tmp_dir EXIT
    set -e  # stop on error
    mkdir -p "$TMP_DIR"

    create_tmp_mkdocs_yml

    # creat subdirectories and hard-link files for live updates
    for f in "$SKEL_DIR"/** ; do
        g="$TMP_DIR/docs/${f#$SKEL_DIR/}"
        if test -d "$f"; then
            mkdir -p "$g"
        # do not copy first-level files
        elif test -f "$f" -a "${SKEL_DIR%/}" != "${f%/*}" ; then
            ln "$f" "$g"  # hard links for live update
        fi
    done

    # move the overrides/ directory one level up
    test -d "$TMP_DIR/docs/overrides" &&
         mv "$TMP_DIR/docs/overrides" "$TMP_DIR/"

    # mandatory files to copy
    local file_list=()
    for item in "${NAV_BEGIN[@]}" "${NAV_END[@]}" ; do
        IFS=${item:0:1} read fname short_title <<< "${item:1}"
        file_list+=("$fname")
    done

    # filter files to copy
    for nav_filter in "${NAV_FILTERS[@]}" ; do
        opt="--${nav_filter%:*}"
        filter=${nav_filter#*:}
        maxn=$(get_maxn_for_opt "$opt")
        for fname in $filter ; do
            n=$(get_number_from_fname "$fname")
            ((maxn < 0 || n <= maxn)) && file_list+=("$fname")
        done
    done

    # create hard links for live update
    echo_col -n "Copying files:"
    for f in "${file_list[@]}" ; do
        test -f "$f" || continue
        echo -n " $f"
        ln "$f" "$TMP_DIR/docs/$f"
    done
    echo

    set +e
}


#----------------------------------- C O R E ----------------------------------

get_local_conf()
{
    test -f "$SKEL_DIR/$LOCAL_CONF" && source "$SKEL_DIR/$LOCAL_CONF"
}


need_local_conf()
{
    test -f "$SKEL_DIR/$LOCAL_CONF" || {
        echo_err "Error: configuration file not found: $SKEL_DIR/$LOCAL_CONF"
        echo_col "To create a skeleton, run: $0 create-skel"
        return 1
    }
}


get_config_hash()
{
    cat "$SKEL_DIR"/{conf-mkdocs-et.sh,mkdocs.yml} | md5sum | tr -d ' -'
}


monitor_config_changes()
{
    local h1=$(get_config_hash) h2

    while true ; do
        sleep "$MONITOR_PERIOD"
        h2=$(get_config_hash)
        if test "$h1" != "$h2" ; then
            echo_col "Changes detected in configuration!"
            h1=$h2
            get_local_conf
            create_tmp_mkdocs_yml  # this will alert mkdocs
        fi
    done
}

run_serve()  # mkdocs_options
{
    local monitor_pid res

    monitor_config_changes &
    monitor_pid=$!

    echo_col "Running web server; interrupt with Ctrl+C..."
    (
        cd "$TMP_DIR" || return 1
        mkdocs serve "$@" | grep -v "^FOUND:"
        res=${PIPESTATUS[0]}     # success status of mkdocs
        kill "$monitor_pid"
        return "$res"
    )
}


apply_tables_fixes()
{
    local f

    echo_col "Applying tables fixes <!-- @noheader,@nosort -->..."
    for f in "$TMP_DIR"/site/**/*.html; do 
        sed -i '/<!-- @noheader -->/,/<\/thead>/c<table class="nosort">' "$f"
        sed -i '/<!-- @nosort -->/,/<table>/c<table class="nosort">' "$f"
    done
}


rename_php_files()
{
    local n=0 f

    echo_col "Renaming files containing PHP code..."
    for f in "$TMP_DIR"/site/**/*.html; do 
        if grep -i "<?php" "$f" >/dev/null 2>&1; then
            mv "$f" "${f%.html}.php" && ((n++))
        fi
    done
    echo_col "$n file(s) renamed."
}


copy_additionnal_files()
{
    cp "$SKEL_DIR/$HTACCESS_SRC" "$TMP_DIR/site/$HTACCESS_DEST"
}


run_build()  # mkdocs_options
{
    echo_col "Generating site ..."
    (
        cd "$TMP_DIR" || return 1
        mkdocs build "$@" | grep -v "^FOUND:"
        ((PIPESTATUS[0] == 0)) || return 1    # success status of mkdocs
    )
    apply_tables_fixes
    rename_php_files
    copy_additionnal_files
}


rsync_site()
{
    local dest="$TARGET_HOST:$TARGET_DIR/" 
    local -l rep  # lowercase

    echo
    if ! $FLAG_FORCE ; then
        echo_col "Check dry rsync on $dest ..."
        echo_col "The following files will be modified on destination:"
        rsync -zaic --delete --dry-run "$TMP_DIR/site/" "$dest" | grep "^[<*]"
        echo_col -n "Do you want to continue? [y/N] "
        read rep
        if ! test "$rep" = "y" -o "$rep" = "yes" ; then
            echo_col "Abort."
            return
        fi
    fi

    # mirror files and delete unwanted files
    echo_col "Mirroring files on $dest ..."
    rsync -zaic --delete "$TMP_DIR/site/" "$dest"

    echo_col "done."
    $FLAG_FORCE || echo_col "Tip: to force the mirroring, add '-f' at the end."
}


#--------------------------- C R E A T E   S K E L ----------------------------

create_file_htaccess()
{
    local f="$SKEL_DIR/htaccess"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
# .htaccess for Mkdocs-et

# Some types to help Apache httpd
AddCharset UTF-8 .html .css .php .js .txt .py .cpp .hpp .c .h
AddType text/plain .txt .py .cpp .hpp .c .h

# This is very important for Search box
AddType application/javascript .js
END_OF_FILE
}


create_file_mkdocs_et_conf()
{
    local f="$SKEL_DIR/conf-mkdocs-et.sh"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
#! /bin/bash

# conf-mkdocs-et.sh - local configuration
#
# This file will be sourced by mkdocs-et.sh
# DO NOT ADD SPACES AROUND "=" !!

# Your login on the target web server
TARGET_HOST="mylogin@myserver.fr"

# Base directory on the target server
TARGET_DIR="~/public_html/mysite"

# The script must be able to connect by ssh without a password; for that, create
# a public key and copy it to the target server (replace TARGET_HOST below):
#   $ test -f ~/.ssh/id_rsa.pub || ssh-keygen -t rsa
#   $ ssh-copy-id -i ~/.ssh/id_rsa.pub TARGET_HOST

# Navigation : mandatory part
# Line format is "|file_name|short_title", 
# where the first char is the separator (here "|").
NAV_BEGIN=(
    "|index.md|Main page"
)

NAV_END=(
  #  "|todo.md|Todo"
  #  "|about.md|About"
)

# Filters for navigation :
# - line format is "option:file_pattern"
# - each option will be available as a command line argument "--option"
# - each file corresponding to file_pattern will be copied on the site
#   (? stands for 1 char and * stands for any string)
# - the number in the file name can be in any position, but not in an extension
NAV_FILTERS=(
  #  'lec:lec-??-*.md'
  #  'prw:prw-??-*.md'
)

# List of titles associated to filtered files.
# Line format is "|file_name|short_title"
# where the first char is the separator (here "|").
NAV_TITLES=(
  #  "|lec-01-basic.md|L 01 - Markdown syntax"
  #  "|lec-02-more.md|L 02 - More Markdown"
  #  "|prw-01-maths.md|PW 01 - Maths"
)
END_OF_FILE
}


create_file_mkdocs_yaml()
{
    local f="$SKEL_DIR/mkdocs.yml"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
# Configuration file for Mkdocs
#
# See:
#   https://www.mkdocs.org
#   https://facelessuser.github.io/pymdown-extensions/
#   https://squidfunk.github.io/mkdocs-material/
#   https://squidfunk.github.io/mkdocs-material-insiders/reference/abbreviations/

# Project information
site_name: Mysite
site_url: http://myserver.fr/~mylogin/mysite
site_author: My name
site_description: My site using Mkdocs-et

# Copyright: for instance CC BY-SA
copyright: CC BY-SA - myemail@myprovider.fr

# Bottom right "Home page" link
extra:
  social:
    - icon: fontawesome/solid/house
      link: http://myserver.fr/~mylogin/

# Configuration
theme:
  name: material
  custom_dir: overrides
  # Select here your language: en, fr, ...
  language: en
  # Select here the colors of your theme, see
  # https://squidfunk.github.io/mkdocs-material/setup/changing-the-colors/#primary-color
  palette:
    primary: 'green'
    # primary: 'brown'          # colors are changed in stylesheets/extra.css
    accent: 'deep orange'
  # Put your favicon.ico file in skel-mkdocs-et/images/, or an URL to a favicon
  # and uncomment the relevant line:
#  favicon: images/favicon.ico
#  favicon: http://myserver.fr/~mylogin/mysite/favicon.ico
  # This is the top left icon
  icon:
    logo: fontawesome/solid/graduation-cap
  features:
#    - navigation.instant       # does no work well with arithmatex
    - navigation.top
    - navigation.footer
#    - toc.integrate            # move the TOC to the left -> enlarge text area
  # Choose the fonts
  font: false                  # forbid Google fonts, use system fonts
#  font:
#    text: Roboto                # Google fonts
#    code: Roboto Mono
#  font:
#    text: 'PT Serif'
#    code: 'Inconsolata'

# Plugins
plugins:
  # Important: for the Search box, using Apache, it is necessary to add 
  #   AddType application/javascript .js
  # in .htaccess ; this is automatically done by mkdocs-et.sh
  - search:
      separator: '[\s\-\.]+'    # Separators for the search box
  # To make diagrams
  - mermaid2
#      arguments:
#        securityLevel: 'loose'
  # Activate jinja2 macros
  # Mind to protect {{ }} {# #} {% %} in between {% raw %} and {% endraw %}
  - macros:
      include_dir: docs         # {% include "includes/filename.j2" %}
  # Plugin to add zoom functionality on images
  - glightbox

# Extensions
markdown_extensions:
  - admonition                  # !!! example "Title"
  - attr_list                   # title { #label data-toc-label="short title" }
  - def_list                    # :   definition
  - footnotes                   # a footnote[^1]
  - md_in_html                  # <div markdown="1">
  - meta                        # header --- key: value---
#  - toc:
#      permalink: true          # show a q/permalink, some bugs
  - pymdownx.arithmatex:        # LaTeX syntax
      generic: true
#  - pymdownx.critic:           # {--deleted--}     DOESN'T WORK
#      mode: view
  - pymdownx.details            # ???+ success "Title"
  - pymdownx.emoji:             # :smile: :heart: :thumbsup:
      emoji_index: !!python/name:material.extensions.emoji.twemoji
      emoji_generator: !!python/name:material.extensions.emoji.to_svg
  - pymdownx.escapeall          # \escape all characters
  - pymdownx.highlight:
      linenums: None            # default. To number: ``` linenums="1"
  - pymdownx.inlinehilite       # `:::language ...`
  - pymdownx.keys               # ++cmd+alt+"&Uuml;"++
  - pymdownx.smartsymbols       # --> <--> =/= 1/4
  - pymdownx.snippets:          # --8<-- "include_file.ext"
      base_path: "docs"
  - pymdownx.superfences:
      preserve_tabs: true       # important in snippets
      # Exception for as mermaid to work, see
      # https://github.com/fralau/mkdocs-mermaid2-plugin
      custom_fences:
        - name: mermaid
          class: mermaid
          format: !!python/name:pymdownx.superfences.fence_code_format
#          format: !!python/name:mermaid2.fence_mermaid
  - pymdownx.tabbed:            # === "Tab 1"
      alternate_style: true
  - pymdownx.tasklist:          # - [X] item
      custom_checkbox: true
  - pymdownx.tilde:             # ~subscript~  ~~strikethrough\ words~~
      smart_delete: false

extra_javascript:
  - javascripts/config.js
  - https://polyfill.io/v3/polyfill.min.js?features=es6
  - https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js
  - https://cdnjs.cloudflare.com/ajax/libs/tablesort/5.2.1/tablesort.min.js
# Useless on recent versions of mkdocs
#  - https://unpkg.com/mermaid/dist/mermaid.min.js

extra_css:
  - stylesheets/extra.css

# To remove annoying error messages in serve mode
# https://github.com/mkdocs/mkdocs/blob/master/docs/user-guide/configuration.md
validation:
  links:
    not_found: ignore
    unrecognized_links: ignore

# Page tree for Navigation menu
# The 'nav:' section MUST BE AT THE END OF THE FILE because mkdocs-et.sh will 
# append sections on the fly, depending on filter options.
# If the 'nav:' line is commented, the script will append one.
#nav:
#  - Main page: 'index.md'

END_OF_FILE
}


create_file_config_js()
{
    local f="$SKEL_DIR/javascripts/config.js"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
// Simple config.js for Mkdocs-et
// This file will be loaded at each page load.

// For mathematical formulas in LaTeX
window.MathJax = {
  tex: {
    inlineMath: [["\\(", "\\)"]],
    displayMath: [["\\[", "\\]"]],
    processEscapes: true,
    processEnvironments: true,
    tags: 'ams'
  },
  options: {
    ignoreHtmlClass: ".*|",
    processHtmlClass: "arithmatex"
  }
};


// tablesort: tables are sortable except those having the "nosort" class
// Disadvantages of nosort:
// - it only works in publish mode, not serve
// - it also remove some table styles (colored header and shadows) 
document$.subscribe(function() {
  var tables = document.querySelectorAll("article table:not(.nosort)");
  tables.forEach(function(table) {
    new Tablesort(table);
  });
});
END_OF_FILE
}


create_file_extra_css()
{
    local f="$SKEL_DIR/stylesheets/extra.css"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
/* extra style for Mkdocs-et */

/* Comment this to go back to the Material's theme colors */

[data-md-color-primary="brown"] {
    --md-primary-fg-color: #615550; 
    --md-footer-bg-color:  #615550; 
    --md-typeset-a-color:  #A9754F; 
    --md-accent-fg-color:  #A9754F; 
}

/* Some other settings, a matter of taste */

a:hover { text-decoration: underline; }

.md-typeset {
    line-height: 1.5 !important;
}
.md-typeset pre {
    line-height: 1.4 !important;
}

.md-typeset code {
    word-break: normal !important;
}

.md-typeset ul ul {
    list-style-type:  circle !important;
}
.md-typeset ul ul ul {
    list-style-type:  '- ' !important;
}

article.md-typeset p + ul {
    margin-top: -0.5em !important;
}

.md-typeset ul li {
    margin-bottom: 0 !important;
}

.md-typeset dl dd {
    margin-top: 0.5em !important;
    margin-bottom: 0 !important;
}

.md-typeset dl dt:not(:first-child) {
    margin-top: 1em !important;
}

.md-typeset .admonition, 
.md-typeset details,
.md-typeset table {
    font-size: inherit !important;
}

.md-typeset table th, .md-typeset table td {
    line-height: 1.4 !important;
}

.md-typeset table th:not(.code), .md-typeset table td:not(.code) {
    padding: 8px 12px !important;
}

.md-typeset table.nosort th {
    font-weight: bold;
}

article input {
    border: 1px solid lightgrey;
    border-radius: 4px;
    padding: 2px 6px;
    margin: 0 4px;
}
END_OF_FILE
}


create_file_main_html()
{
    local f="$SKEL_DIR/overrides/main.html"
    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'
{% extends "base.html" %}

<!-- Add extra copyright at the page bottom. The extracopyright variable has
     been added in material/templates/partials/copyright.html by Mkdocs-et -->
{% set extracopyright %}
and <a href="https://pageperso.lis-lab.fr/~edouard.thiel/mkdocs-et/">Mkdocs-et</a>
{% endset %}

<!-- Override the navigation title. The nav_title variable has been added 
     in material/templates/partials/nav.html by Mkdocs-et -->
{% set nav_title %}
Navigation
{% endset %}
END_OF_FILE
}


create_file_index()
{
    local f="index.md"

    if test -f "$f" ; then
        echo_col "  = $f already exists"
        return
    fi

    echo_col "  + $f"
    cat >| "$f" << 'END_OF_FILE'

# My new project

## 1. Welcome to my site


Hello world!


## 2. Documentation

See the documentation of [Mkdocs-et][1].

[1]: https://pageperso.lis-lab.fr/~edouard.thiel/mkdocs-et/

END_OF_FILE
}


create_skel()
{
    local d

    test -d "$SKEL_DIR" && {
        echo_err "Error: $SKEL_DIR already exists"
        return 1
    }

    set -e
    echo_col "Creating skeleton directory $SKEL_DIR..."
    mkdir "$SKEL_DIR"

    for d in images includes javascripts stylesheets overrides ; do
        echo_col "  + $SKEL_DIR/$d/"
        mkdir "$SKEL_DIR/$d"
    done

    create_file_htaccess
    create_file_mkdocs_et_conf
    create_file_mkdocs_yaml
    create_file_config_js
    create_file_extra_css
    create_file_main_html
    create_file_index

    set +e
    echo_col "Skeleton directory $SKEL_DIR created."
}


#----------------------------- A R G U M E N T S ------------------------------

show_options()
{
    local s="[--all]" f

    if ! declare -p NAV_FILTERS >/dev/null 2>&1 ; then
        echo "$s [filters]"
        return
    fi

    for f in "${NAV_FILTERS[@]}" ; do
        s+=" [--${f%%:*} n]"
    done
    echo "$s"
}


parse_args() # remain_args "$@"
{
    # prefix namerefs with '_' to have distinct names
    local -n _remain_args=$1 ; shift 1
    local f serve_port=""

    for f in "${NAV_FILTERS[@]}" ; do
        ARG_FILTERS[--${f%%:*}]="0"
    done

    _remain_args=()
    while (($# > 0)); do
        test -z "$1" && continue
        if test "$1" = "--" ; then
            shift ; break
        elif test "$1" = "--all" ; then
            FLAG_ALL=true; shift
        elif test "$1" = "-f" ; then
            FLAG_FORCE=true ; shift
        elif test "$1" = "-p" ; then
            is_integer "$2" || {
                echo_err "Error: expected integer after $1"
                exit 1
            }
            serve_port=$2
            shift 2
        elif test -n "${ARG_FILTERS[$1]}" ; then
            is_integer "$2" || {
                echo_err "Error: expected integer after $1"
                exit 1
            }
            ARG_FILTERS[$1]=$2
            shift 2
        else
            _remain_args+=("$1")
            shift
        fi
    done

   _remain_args+=("$@")
    test -n "$serve_port" && _remain_args+=(--dev-addr "localhost:$serve_port")
}


show_args() # remain_args
{
    # prefix namerefs with '_' to have distinct names
    local -n _remain_args=$1
    local k

    echo_col -n "Filters:"
    $FLAG_ALL && echo -n " --all"
    for k in "${!ARG_FILTERS[@]}"; do echo -n " $k ${ARG_FILTERS[$k]}"; done
    echo " ; remaining args passed to mkdocs: ${_remain_args[@]}"
}


#---------------------------------- U S A G E ---------------------------------

show_title()
{
    cat <<- EOT
	---------------------------------------------------------------------------
	  mkdocs-et.sh  V $VERSION
	  https://pageperso.lis-lab.fr/~edouard.thiel/mkdocs-et/
	---------------------------------------------------------------------------
	
	EOT
}


show_usage()
{
    local options=$(show_options)
    cat <<- EOT
	USAGE:
	    ${0##*/} action
	action:
	    create-venv|remove-venv|update-venv|list-venv
	    create-skel
	    serve   $options [-p port] [--] [mkdocs options]
	    publish $options [-f]      [--] [mkdocs options]
	
	EOT
}


#----------------------------------- M A I N ----------------------------------

show_title
get_local_conf
show_usage

check_bash_version || exit 1

shopt -s globstar       # allow pattern **

action=""
case "$1" in
    create-venv) create_venv ; exit ;;
    remove-venv) remove_venv ; exit ;;
    update-venv) remove_venv ; create_venv ; exit ;;
    list-venv) run_venv && list_venv ; exit ;;
    create-skel) create_skel ; exit ;;
    serve|publish) action="$1" ; shift ;;
    "") echo_err "Warning: action expected" ; exit 1 ;;
    *) echo_err "Error: unknown action" ; exit 1 ;;
esac

need_local_conf || exit 1

declare -A ARG_FILTERS
FLAG_ALL=false
FLAG_FORCE=false
parse_args remain_args "$@"
show_args  remain_args
set -- "${remain_args[@]}"

case "$action" in
    serve)   run_venv && create_tmp_dir && run_serve "$@" ;;
    publish) run_venv && create_tmp_dir && run_build "$@" && rsync_site ;;
esac


